@@ -12,7 +12,8 @@ description = "cargo auto - automation tasks written in Rust language"
 publish = false
 
 [dependencies]
-cargo_auto_lib = "0.7.8"
+cargo_auto_lib = "0.7.15"
+tokio = {version = "1.10.0", features = ["rt","rt-multi-thread","fs"]}
 "#
 }
 
@@ -58,8 +59,13 @@ fn match_arguments_and_call_tasks(mut args: std::env::Args) {
                     task_increment_minor();
                 } else if &task == "docs" || &task == "doc" || &task == "d" {
                     task_docs();
+                } else if &task == "commit_and_push" {
+                    let arg_2 = args.next();
+                    task_commit_and_push(arg_2);
                 } else if &task == "publish_to_crates_io" {
                     task_publish_to_crates_io();
+                } else if &task == "github_new_release" {
+                    task_github_new_release();
                 } else {
                     println!("Task {} is unknown.", &task);
                     print_help();
@@ -78,7 +84,12 @@ cargo auto build - builds the crate in debug mode, fmt
 cargo auto release - builds the crate in release mode, version from date, fmt
 cargo auto increment_minor - increments the semver version minor part (only for libraries)
 cargo auto docs - builds the docs, copy to docs directory
+cargo auto commit_and_push - commits with message and push with mandatory message
+    if you use SSH, it is easy to start the ssh-agent in the background and ssh-add your credentials for git
 cargo auto publish_to_crates_io - publish to crates.io, git tag
+cargo auto github_new_release - creates new release on github
+    this task needs PAT (personal access token from github) in the env variable: `export GITHUB_TOKEN=paste_token_here`
+
 "#
     );
 }
@@ -90,7 +101,7 @@ fn completion() {
     let last_word = args[3].as_str();
 
     if last_word == "cargo-auto" || last_word == "auto" {
-        let sub_commands = vec!["build", "release", "doc", "publish_to_crates_io"];
+        let sub_commands = vec!["build", "release", "doc","commit_and_push","publish_to_crates_io","github_new_release"];
         completion_return_one_or_more_sub_commands(sub_commands, word_being_completed);
     }
     /*
@@ -115,7 +126,8 @@ fn task_build() {
     run_shell_commands(shell_commands.to_vec());
     println!(
         r#"
-After `cargo auto build`, run the tests and the code. If ok, then `cargo auto release`
+After `cargo auto build`, run the tests and the code. If ok, then 
+run`cargo auto release`
 "#
     );
 }
@@ -134,7 +146,8 @@ fn task_release() {
 
     println!(
         r#"
-After `cargo auto release`, run the tests and the code. If ok, then `cargo auto doc`
+After `cargo auto release`, run the tests and the code. If ok, then 
+run `cargo auto doc`
 "#
     );
 }
@@ -160,12 +173,29 @@ fn task_docs() {
     // message to help user with next move
     println!(
         r#"
-After `cargo auto doc`, check `docs/index.html`. If ok, then `git commit -am"message"` and `git push`,
-then `cargo auto publish_to_crates_io`
+After `cargo auto doc`, check `docs/index.html`. If ok, then 
+run `cargo auto commit_and_push` with mandatory commit message
 "#
     );
 }
 
+/// commit and push
+fn task_commit_and_push(arg_2: Option<String>) {
+    match arg_2 {
+        None => println!("Error: message for commit is mandatory"),
+        Some(message) => {
+            run_shell_command(&format!(r#"git add -A && git commit -m "{}""#, message));
+            run_shell_command("git push");
+            println!(
+                r#"
+After `cargo auto commit and push`
+run `cargo auto publish_to_crates_io` or `cargo auto github_new_release`
+"#
+            );
+        }
+    }
+}
+
 /// example hot to publish to crates.io and git tag
 fn task_publish_to_crates_io() {
     // git tag
@@ -179,46 +209,63 @@ fn task_publish_to_crates_io() {
     run_shell_command("cargo publish");
     println!(
         r#"
-After `cargo auto task_publish_to_crates_io', check `crates.io` page.
-If binary then install with `cargo install crate_name` and check how it works.
-If library then add dependency to your rust project and check how it works.
-"#
+After `cargo auto task_publish_to_crates_io', 
+check `https://crates.io/crates/{package_name}`.
+If binary then install with `cargo install {package_name}` and check how it works.
+If library then add dependency `{package_name} = "{package_version}"` to your rust project and check how it works.
+"#,
+        package_name = package_name(),
+        package_version = package_version()
     );
 }
 
+/// create a new release on github with octocrab
+/// the env variable GITHUB_TOKEN must be set `export GITHUB_TOKEN=paste_token_here`
+fn task_github_new_release() {
+    // async block inside sync code with tokio
+    use tokio::runtime::Runtime;
+    let rt = Runtime::new().unwrap();
+    rt.block_on(async move {
+        let owner = github_owner();
+        let repo = package_name();
+        let version = package_version();
+        let name = format!("Version {}", &package_version());
+        let branch = "main";
+        let body_md_text = &format!(
+            r#"
+This is {package_name} from {owner}.
+Write something meaningful here. Maybe instructions how to install.
+Or maybe what changed in this release.
+            "#,
+            package_name = package_name(),
+            owner = owner
+        );
+
+        let release_id =  github_create_new_release(&owner, &repo, &version, &name, branch, body_md_text).await;
+        println!("New release created, now uploading release asset. This can take some time if the files are big. Wait ...");
+
+        // upload asset
+        let path_to_file = format!(
+            "target/release/{package_name}",
+            package_name = package_name()
+        );
+
+        github_upload_asset_to_release(&owner, &repo, &release_id, &path_to_file).await;
+        println!(
+            r#"
+After `cargo auto github_new_release', 
+check `https://github.com/{github_owner}/{package_name}/releases/tag/v{package_version}`.
+Download it and check hot it works.  
+"#,
+            package_name = package_name(),
+            package_version = package_version(),
+            github_owner = github_owner(),
+        );
+    });
+}
+
 // endregion: tasks
 
-// region: helper functions
-
-/// check if run in rust project root directory error
-/// there must be Cargo.toml and the directory automation_tasks_rs
-/// exit with error message if not
-fn exit_if_not_run_in_rust_project_root_directory() {
-    if !(std::path::Path::new("automation_tasks_rs").exists() && std::path::Path::new("Cargo.toml").exists()) {
-        println!("Error: automation_tasks_rs must be called in the root directory of the rust project beside the Cargo.toml file and automation_tasks_rs directory.");
-        // early exit
-        std::process::exit(0);
-    }
-}
-
-/// println one, more or all sub_commands
-fn completion_return_one_or_more_sub_commands(sub_commands: Vec<&str>, word_being_completed: &str) {
-    let mut sub_found = false;
-    for sub_command in sub_commands.iter() {
-        if sub_command.starts_with(word_being_completed) {
-            println!("{}", sub_command);
-            sub_found = true;
-        }
-    }
-    if sub_found == false {
-        // print all sub-commands
-        for sub_command in sub_commands.iter() {
-            println!("{}", sub_command);
-        }
-    }
-}
-
-// endregion: helper functions
 
     "##
 }
